#Soruce: https://github.com/antistatique/swisstopo/blob/dev/src/SwisstopoConverter.php

class SwisstopoConverter:

    @staticmethod
    def from_mn95_to_wgs(east: float, north: float) -> dict:
        return {
            'lat': SwisstopoConverter.from_mn95_to_wgs_latitude(east, north),
            'long': SwisstopoConverter.from_mn95_to_wgs_longitude(east, north),
        }

    @staticmethod
    def from_wgs_to_mn95(lat: float, lon: float) -> dict:
        return {
            'east': SwisstopoConverter.from_wgs_to_mn95_east(lat, lon),
            'north': SwisstopoConverter.from_wgs_to_mn95_north(lat, lon),
        }

    @staticmethod
    def from_mn03_to_wgs(y: int, x: int) -> dict:
        return {
            'lat': SwisstopoConverter.from_mn03_to_wgs_latitude(y, x),
            'long': SwisstopoConverter.from_mn03_to_wgs_longitude(y, x),
        }

    @staticmethod
    def from_wgs_to_mn03(lat: float, lon: float) -> dict:
        return {
            'x': SwisstopoConverter.from_wgs_to_mn03x(lat, lon),
            'y': SwisstopoConverter.from_wgs_to_mn03y(lat, lon),
        }

    @staticmethod
    def from_wgs_to_mn03y(lat: float, lon: float) -> float:
        lat = SwisstopoConverter.deg_to_sex(lat)
        lon = SwisstopoConverter.deg_to_sex(lon)

        lat = SwisstopoConverter.deg_to_sec(lat)
        lon = SwisstopoConverter.deg_to_sec(lon)

        lat_aux = (lat - 169028.66) / 10000
        lon_aux = (lon - 26782.5) / 10000

        return 600072.37 + 211455.93 * lon_aux - 10938.51 * lon_aux * lat_aux - 0.36 * lon_aux * lat_aux**2 - 44.54 * lon_aux**3

    @staticmethod
    def from_wgs_to_mn03x(lat: float, lon: float) -> float:
        lat = SwisstopoConverter.deg_to_sex(lat)
        lon = SwisstopoConverter.deg_to_sex(lon)

        lat = SwisstopoConverter.deg_to_sec(lat)
        lon = SwisstopoConverter.deg_to_sec(lon)

        lat_aux = (lat - 169028.66) / 10000
        lon_aux = (lon - 26782.5) / 10000

        return 200147.07 + 308807.95 * lat_aux + 3745.25 * lon_aux**2 + 76.63 * lat_aux**2 - 194.56 * lon_aux**2 * lat_aux + 119.79 * lat_aux**3

    @staticmethod
    def from_wgs_to_mn95_north(lat: float, lon: float) -> float:
        lat = SwisstopoConverter.deg_to_sex(lat)
        lon = SwisstopoConverter.deg_to_sex(lon)

        phi = SwisstopoConverter.deg_to_sec(lat)
        lambda_ = SwisstopoConverter.deg_to_sec(lon)

        phi_aux = (phi - 169028.66) / 10000
        lambda_aux = (lambda_ - 26782.5) / 10000

        return 1200147.07 + 308807.95 * phi_aux + 3745.25 * lambda_aux**2 + 76.63 * phi_aux**2 - 194.56 * lambda_aux**2 * phi_aux + 119.79 * phi_aux**3

    @staticmethod
    def from_wgs_to_mn95_east(lat: float, lon: float) -> float:
        lat = SwisstopoConverter.deg_to_sex(lat)
        lon = SwisstopoConverter.deg_to_sex(lon)

        phi = SwisstopoConverter.deg_to_sec(lat)
        lambda_ = SwisstopoConverter.deg_to_sec(lon)

        phi_aux = (phi - 169028.66) / 10000
        lambda_aux = (lambda_ - 26782.5) / 10000

        return 2600072.37 + 211455.93 * lambda_aux - 10938.51 * lambda_aux * phi_aux - 0.36 * lambda_aux * phi_aux**2 - 44.54 * lambda_aux**3

    @staticmethod
    def from_mn95_to_wgs_latitude(east: float, north: float) -> float:
        y_aux = (east - 2600000) / 1000000
        x_aux = (north - 1200000) / 1000000

        lat = 16.9023892 + 3.238272 * x_aux - 0.270978 * y_aux**2 - 0.002528 * x_aux**2 - 0.0447 * y_aux**2 * x_aux - 0.0140 * x_aux**3
        lat = lat * 100 / 36

        return lat

    @staticmethod
    def from_mn95_to_wgs_longitude(east: float, north: float) -> float:
        y_aux = (east - 2600000) / 1000000
        x_aux = (north - 1200000) / 1000000

        long = 2.6779094 + 4.728982 * y_aux + 0.791484 * y_aux * x_aux + 0.1306 * y_aux * x_aux**2 - 0.0436 * y_aux**3
        long = long * 100 / 36

        return long

    @staticmethod
    def from_mn03_to_wgs_latitude(y: float, x: float) -> float:
        y_aux = (y - 600000) / 1000000
        x_aux = (x - 200000) / 1000000

        lat = 16.9023892 + 3.238272 * x_aux - 0.270978 * y_aux**2 - 0.002528 * x_aux**2 - 0.0447 * y_aux**2 * x_aux - 0.0140 * x_aux**3
        lat = lat * 100 / 36

        return lat

    @staticmethod
    def from_mn03_to_wgs_longitude(y: float, x: float) -> float:
        y_aux = (y - 600000) / 1000000
        x_aux = (x - 200000) / 1000000

        long = 2.6779094 + 4.728982 * y_aux + 0.791484 * y_aux * x_aux + 0.1306 * y_aux * x_aux**2 - 0.0436 * y_aux**3
        long = long * 100 / 36

        return long

    @staticmethod
    def deg_to_sex(angle: float) -> float:
        deg = int(angle)
        min = int((angle - deg) * 60)
        sec = (((angle - deg) * 60) - min) * 60

        return deg + min / 100 + sec / 10000

    @staticmethod
    def deg_to_sec(angle: float) -> float:
        deg = int(angle)
        min = int((angle - deg) * 100)
        sec = (((angle - deg) * 100) - min) * 100

        return sec + min * 60 + deg * 3600